/* Therm2 PIC16F628*/

#include <xc.h>
#include <stdlib.h>
#include <string.h>

#pragma   config FOSC = INTOSCIO,  MCLRE = ON
#pragma   config PWRTE = ON, BOREN = OFF
#pragma   config  WDTE = OFF
#pragma   config  LVP = OFF
#pragma   config  CP = OFF
#pragma   config  CPD = OFF

#define ROW TRISBbits.TRISB0=1;
#define TOW TRISBbits.TRISB0=0;


static char s1[] = "\x0F\x0E\x0D\x0D"; //C
static char s2[] = "\x0C\x0C\x0C\x0C"; //----
static char s3[] = "\x0A\x0B\x0D\x0D"; //AL

char alarm[4];
char str2[8] = "   ";
char str[8] = "   ";
char tf = 0;
char tlac = 0, otla = 0;
unsigned int count = 0;
unsigned int tms = 0;
char sp = 0;
char menu = 0;
char dt = 2;
char fire = 0;
int alrm = 950; //alarm 95 C
int term = 0;
int tread = 0x0191; //25 C
int inca[4] = {1000, 100, 10, 1};

char segm[] = {
    //    RB  76543210     RB0 = data OW
    //        decgbfaw
    0x11, //  00010001 	0
    0xD7, //  11010111 	1
    0x61, //  01100001 	2
    0x45, //  01000101 	3
    0x87, //  10000111 	4
    0x0D, //  00001101 	5
    0x09, //  00001001 	6
    0xD5, //  11010101 	7	
    0x01, //  00000001 	8
    0x05, //  00000101  9
    0x81, //  10000001  A        A
    0x3B, //  00111011  L        B
    0xEF, //  11101111  -        C
    0xFF, //  11111111  mezera   D
    0x39, //  00111001  C        E
    0xA5, //  10100101          F
};

void Delay(unsigned int dms) {
    tms = dms;
    while (tms);
};

void ItoAscii(int val) {
    char len;

    itoa(str2, val, 10);
    len = (char) strlen((const char *) str2);
    if (str2[0] == '-')str2[0] = 0x0C;

    if (len == 3) {
        str2[3] = str2[2];
        str2[2] = str2[1];
        str2[1] = str2[0];
        str2[0] = 0x0D;
    }
    if (len == 2) {
        str2[3] = str2[1];
        str2[2] = str2[0];
        str2[1] = str2[0] = 0x0D;
    }
    if (len == 1) {
        str2[3] = str2[0];
        str2[2] = str2[1] = str2[0] = 0x0D;
        if (val < 10)str2[2] = '0';
    }
    strncpy(str, (const char*) str2, 4);
};

char ow_detect(void) {
    char out = 1;

    TOW;
    _delay(480);
    ROW;
    _delay(70);
    if (PORTB & 0x01) out = 0;
    _delay(410);
    return out;
};

void ow_write_one(void) {
    TOW;
    _delay(6);
    ROW;
    _delay(64);
};

void ow_write_zero(void) {
    TOW;
    _delay(60);
    ROW;
    _delay(10);
};

char ow_read_bit(void) {
    char out = 0;
    TOW;
    _delay(6);
    ROW;
    _delay(9);
    if (PORTB & 0x01) out = 1;
    _delay(55);
    return out;
};

void ow_write_byte(char tosend) {
    char n = 8;
    while (n--) {
        if (tosend & 1) ow_write_one();
        else ow_write_zero();
        tosend >>= 1;
    }
};

char ow_read_byte(void) {
    char n = 8, out = 0;
    while (n--) {
        out >>= 1;
        if (ow_read_bit()) out |= 0x80;
    }
    return out;
};

void temp_init() {

    GIE = 0;
    PORTA &= 0xF0;
    PORTB &= 0xFE; //RB0 =0;
    ow_detect();
    ow_write_byte(0xCC);
    ow_write_byte(0x44);
    GIE = 1;
};

void temp_read() {
    char data_lo;
    char data_hi;
    GIE = 0;
    PORTA &= 0xF0;
    PORTB &= 0xFE; //RB0 =0;  
    ow_detect();
    ow_write_byte(0xCC);
    ow_write_byte(0xBE);
    data_lo = ow_read_byte();
    data_hi = ow_read_byte();
    tread = data_lo;
    *((signed char *) &tread + 1) = data_hi;
    GIE = 1;
};

void Readeprom() {
    alrm = eeprom_read(0);
    *((char *) &alrm + 1) = eeprom_read(1);
}

void Ewrite() {

    eeprom_write(0, (char) alrm);
    eeprom_write(1, *((char *) &alrm + 1));
}

void main(void) {

    PORTB = 0;
    PORTA = 0;
    TRISA = 0x20; //RA5 mclr
    TRISB = 0x01; //RB0 je serial
    OPTION_REG = 0x06; // internal clk, presc. 128
    T2CON = 0x4D; //01001101   
    PR2 = 24;
    TMR2IE = 1;
    INTCON = 0xC0; // PEIE, GEI

    strcpy(str, (const char *) s1);
    dt = 4;
    Readeprom();

    PORTA &= 0xF0;
    PORTB &= 0xFE; //RB0 =0;  
    GIE = 0;
    if (ow_detect() == 0) {
        GIE = 1;
        strcpy(str, (const char *) s2);
        Delay(1600);
    }

    temp_init();
    Delay(3200);
    dt = 2;
    while (1) {

        if (otla == 0 && tlac & 0x0E) {
            otla = 1;

            if (tlac & 0x02) {
                if (menu == 0) {
                    strcpy(str, (const char *) s3);
                    dt = 4;
                    Delay(1600);
                    menu = 1;
                    sp = 0;
                    dt = 2;
                } else {
                    sp++;
                    if (sp > 3) {
                        sp = 0;
                        menu = 0;
                        dt = 4;
                        strcpy(str, (const char *) s1);
                        Ewrite();
                        Delay(1600);
                        dt = 2;
                    }
                }
            }
            if (menu == 1) {
                if (tlac & 0x08)alrm += inca[sp];
                else if (tlac & 0x04)alrm -= inca[sp];
                if (alrm > 9999)alrm = 9999;
                if (alrm<-999)alrm = -999;
                ItoAscii(alrm);
            }
        }

        if ((tlac & 0x0E) == 0)otla = 0;

        if (menu == 0) {

            if (tf) {
                temp_read();
                temp_init();
                //   tread = 0xFFF0;
                term = tread * 5 / 8;
                ItoAscii(term);
                tf = 0;
                count = 1600;
            }
        }
        GIE = 0;
        if (term >= alrm) {
            fire = 1;
        } else {
            fire = 0;
        }
        GIE = 1;
    };
} //main

void interrupt myIsr(void) {
    static char psh = 1, p = 0;
    static char ch, fl = 0, fi;
    if (TMR2IF) {
        TMR2IF = 0;
        if (fire) {
            fi = 0x80;
            if (p & 1) fi |= 0x40;
        } else fi = 0x20;
        PORTA = fi;

        TRISB |= 0x0E;
        ch = (char) (str[p]&0x0F);
        tlac = (char) ~PORTB;
        TRISB &= ~0x0E;
        PORTB = segm[ch];

        if (p == dt)ch = 0;
        else ch = 0x10;
        if (fl & 0x20 && ((menu == 1 && p == sp) || fire));
        else PORTA = (char) (psh | ch | fi);

        psh <<= 1;
        p++;
        if (p == 4) {
            p = 0;
            psh = 1;
            fl++;
        }
        if (tms)tms--;
        if (count)count--;
        if (count == 0) {
            tf = 1;
        }
    } //if TMR2  
}; // Isr

/*
void zmer(){
  char data_lo;
  char data_hi; 
  GIE=0;
  PORTA &=0xF0;
  PORTB &=0xEF; //RB4 =0;
  ow_detect();
  ow_write_byte(0xCC);
  ow_write_byte(0x44);
  delay_ms(300);
  ow_detect();
  ow_write_byte(0xCC);
  ow_write_byte(0xBE);
  data_lo=ow_read_byte();
  data_hi=ow_read_byte();           
  //tread=data_lo  + (data_hi<<8);
  tread=data_lo;
 *((signed char *)&tread +1)=  data_hi;
  
  GIE=1;  
};*/